﻿using System;
using System.Collections.Generic;
using System.Xml;
using System.Xml.Linq;
using System.Xml.XPath;

namespace XmlTestApp
{
  public class BindableXNode
  {
    XPathNavigator navigator;
    IXmlNamespaceResolver resolver;

    public BindableXNode(XNode node, IXmlNamespaceResolver resolver = null)
    {
      this.navigator = node.CreateNavigator();
      this.resolver = resolver;
    }
    private BindableXNode(XPathNavigator navigator, IXmlNamespaceResolver resolver)
    {
      this.navigator = navigator.Clone();
      this.resolver = resolver;
    }
    public object this[string indexer]
    {
      get
      {
        object result = this.navigator.Evaluate(indexer, this.resolver);

        if ((result is bool) || (result is string) || (result is double))
        {
          return (result);
        }
        else
        {
          XPathNodeIterator iterator = (XPathNodeIterator)result;

          List<BindableXNode> items = new List<BindableXNode>();

          while (iterator.MoveNext())
          {
            items.Add(new BindableXNode(iterator.Current,
              this.resolver));
          }
          return (items);
        }
      }
      set
      {
        string stringValue = value.ToString();

        if (this.navigator.UnderlyingObject is XElement)
        {
          XElement xElement = (XElement)this.navigator.UnderlyingObject;
          xElement.Value = stringValue;
        }
        else if (this.navigator.UnderlyingObject is XAttribute)
        {
          XAttribute xAttribute = (XAttribute)this.navigator.UnderlyingObject;
          xAttribute.Value = stringValue;
        }
        else
        {
          throw new NotSupportedException();
        }
      }
    }
  }
}