﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Windows.Markup;
using System.Xml;
using System.Xml.Linq;

namespace XmlTestApp
{
  [ContentProperty("NamespaceMappings")]
  public class XmlDataSource : PropertyChangeNotification
  {
    public XmlDataSource()
    {
      this.NamespaceMappings = new List<XmlNamespaceMapping>();
    }
    public List<XmlNamespaceMapping> NamespaceMappings { get; set; }

    public Uri ResourceUri
    {
      get
      {
        return (_ResourceUri);
      }
      set
      {
        _ResourceUri = value;
        Validate();
        base.RaisePropertyChanged("ResourceUri");
        LoadXml();
      }
    }
    Uri _ResourceUri;

    public Uri DownloadUri
    {
      get
      {
        return (_DownloadUri);
      }
      set
      {
        _DownloadUri = value;
        Validate();
        base.RaisePropertyChanged("DownloadUri");
        LoadXml();
      }
    }
    Uri _DownloadUri;

    public BindableXNode Xml
    {
      get
      {
        return (_Xml);
      }
      internal set
      {
        _Xml = value;
        base.RaisePropertyChanged("Xml");
      }
    }
    BindableXNode _Xml;

    void Validate()
    {
      if ((this.DownloadUri != null) && (this.ResourceUri != null))
      {
        throw new InvalidOperationException(
          "Cannot set both the resource and the download Uri");
      }
    }
    void LoadXml()
    {
      if (this.ResourceUri != null)
      {
        using (XmlReader reader = XmlReader.Create(
          this.ResourceUri.ToString()))
        {
          ReadXml(reader);
          reader.Close();
        }
      }
      else if (this.DownloadUri != null)
      {
        WebClient webClient = new WebClient();

        webClient.OpenReadCompleted += (s, e) =>
        {
          if (e.Error == null)
          {
            using (XmlReader reader = XmlReader.Create(e.Result))
            {
              ReadXml(reader);
              reader.Close();
            }
          }
        };

        webClient.OpenReadAsync(this.DownloadUri);
      }
    }
    void ReadXml(XmlReader reader)
    {
      XElement xElement = XElement.Load(reader);

      if (this.NamespaceMappings.Count > 0)
      {
        XmlNamespaceManager manager = new XmlNamespaceManager(
          new NameTable());

        foreach (var item in this.NamespaceMappings)
        {
          manager.AddNamespace(item.Prefix, item.Namespace);
        }
        this.Xml = new BindableXNode(xElement, manager);
      }
      else
      {
        this.Xml = new BindableXNode(xElement);
      }
    }
  }
}
